////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <Stdio.H>    
#include <Stdlib.H> 
#include <String.H> 

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "../SharedClasses/LZARI/Compression.H"
#include "../SharedClasses/LZARI/Inflate.H"
#include "../SharedClasses/LZARI/Deflate.H"

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define BUFFER_SIZE (1024 * 1024)
#define EXTRA_SIZE  (1024 * 1024)

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CompressFile(char *sSource, char *sTarget)
{
    HANDLE hfSource = NULL;
    HANDLE hfTarget = NULL;

    hfSource = CreateFile(
        sSource,
        GENERIC_READ,
        FILE_SHARE_READ,
        NULL,
        OPEN_EXISTING,
        FILE_ATTRIBUTE_NORMAL,
        NULL
    );

    if(hfSource == INVALID_HANDLE_VALUE)
    {
		MessageBox(NULL, "LZARI:: Failed to open source file.", NULL, 0);
        return 0;
    }

    hfTarget = CreateFile(
        sTarget,
        GENERIC_WRITE,
        0,
        NULL,
        CREATE_ALWAYS,
        FILE_ATTRIBUTE_NORMAL,
        NULL
    );

    if(hfSource == INVALID_HANDLE_VALUE)
    {
        MessageBox(NULL, "LZARI:: Failed to open target file.", NULL, 0);
        CloseHandle(hfSource);
        return 0;
    }

    LZARI MyLZARI;

    MyLZARI.Initialize();

    DWORD BytesRead = 0;
    DWORD BytesWritted = 0;
    DWORD DeflSize = 0;

    char *SourceBuf = NULL;
    char *TargetBuf = NULL;

    SourceBuf = (char *) calloc(BUFFER_SIZE + 1, sizeof(char));
    TargetBuf = (char *) calloc((BUFFER_SIZE + EXTRA_SIZE) + 1, sizeof(char));

    do{
        ReadFile(hfSource, SourceBuf, BUFFER_SIZE, &BytesRead, NULL);

        DeflSize = (BUFFER_SIZE + EXTRA_SIZE);

        MyLZARI.Deflate(SourceBuf, TargetBuf, BytesRead, &DeflSize);
        //printf("%d Bytes to %d Bytes\n", BytesRead, DeflSize);

        WriteFile(hfTarget, &BytesRead, sizeof(DWORD), &BytesWritted, NULL);
        WriteFile(hfTarget, &DeflSize, sizeof(DWORD), &BytesWritted, NULL);
        WriteFile(hfTarget, TargetBuf, DeflSize, &BytesWritted, NULL);
    } while(BytesRead == BUFFER_SIZE);

    free(SourceBuf);
    free(TargetBuf);

    CloseHandle(hfSource);
    CloseHandle(hfTarget);

    return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int DecompressFile(char *sSource, char *sTarget)
{
    HANDLE hfSource = NULL;
    HANDLE hfTarget = NULL;

    hfSource = CreateFile(
        sSource,
        GENERIC_READ,
        FILE_SHARE_READ,
        NULL,
        OPEN_EXISTING,
        FILE_ATTRIBUTE_NORMAL,
        NULL
    );

    if(hfSource == INVALID_HANDLE_VALUE)
    {
        MessageBox(NULL, "LZARI:: Failed to open source file.", NULL, 0);
        return 0;
    }

    hfTarget = CreateFile(
        sTarget,
        GENERIC_WRITE,
        0,
        NULL,
        CREATE_ALWAYS,
        FILE_ATTRIBUTE_NORMAL,
        NULL
    );

    if(hfSource == INVALID_HANDLE_VALUE)
    {
        MessageBox(NULL, "LZARI:: Failed to open target file.", NULL, 0);
        CloseHandle(hfSource);
        return 0;
    }

    LZARI MyLZARI;

    MyLZARI.Initialize();

    char *SourceBuf = NULL;
    char *TargetBuf = NULL;

    SourceBuf = (char *) calloc((BUFFER_SIZE + EXTRA_SIZE), sizeof(char));
    TargetBuf = (char *) calloc((BUFFER_SIZE + EXTRA_SIZE), sizeof(char));

    DWORD OrigSize = 0;
    DWORD DeflSize = 0;
    DWORD InflSize = 0;
    DWORD BytesRead = 0;
    DWORD BytesWritted = 0;

    do{
        ReadFile(hfSource, &OrigSize, sizeof(DWORD), &BytesRead, NULL);
        ReadFile(hfSource, &DeflSize, sizeof(DWORD), &BytesRead, NULL);
        ReadFile(hfSource, SourceBuf, DeflSize, &BytesRead, NULL);

        InflSize = (BUFFER_SIZE + EXTRA_SIZE);

        MyLZARI.Inflate(SourceBuf, TargetBuf, OrigSize, &InflSize);
        //printf("%d Bytes to %d Bytes\n", DeflSize, InflSize);

        WriteFile(hfTarget, TargetBuf, InflSize, &BytesWritted, NULL);
    } while(OrigSize == BUFFER_SIZE);

    free(SourceBuf);
    free(TargetBuf);

    CloseHandle(hfSource);
    CloseHandle(hfTarget);

    return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
